class BenchmarkConfiguration < ActiveRecord::Base

  has_many :benchmark_batches, dependent: :destroy, inverse_of: :benchmark_configuration
  belongs_to :language, inverse_of: :benchmark_configurations
  belongs_to :runtime, inverse_of: :benchmark_configurations
  belongs_to :operating_system, inverse_of: :benchmark_configurations

  scope :with_preloaded_environment_description, -> do
    includes(:operating_system, :runtime, :language)
  end

  def self.from_json(data)
    lang = find_or_create_by_name(Language, data["language"])
    runtime = find_or_create_by_name(lang.runtimes, data["runtime"])
    os = find_or_create_by_name(OperatingSystem, data["os"])
    desc = data["description"] || ""

    configuration_params = { language: lang,
                             runtime: runtime,
                             operating_system: os,
                             description: desc }
    configuration = find_by(configuration_params)
    configuration = create!(configuration_params) unless configuration

    configuration
  end

  def self.find_for_results_json(configuration_id)
    includes(:benchmark_batches).find(configuration_id)
  end

  def self.for_benchmark_infos(info_ids)
    joins(:benchmark_batches).
      joins(benchmark_batches: :benchmark_results).
      where(benchmark_results: { benchmark_info_id: info_ids }).
      distinct.
      with_preloaded_environment_description
  end

  def name
    xlated_os = operating_system.name_normalized
    full_name = "#{runtime.name} (#{language.name}) on #{xlated_os}"

    if description.present?
      "#{description} - #{full_name}"
    else
      full_name
    end
  end

  def all_dates_when_performed
    SortedSet.new(benchmark_batches.map(&:performed_at).flatten).to_a
  end

  # Return timestamps and durations for each benchmark info ever
  # recorded for this configuration
  #
  # @return
  #   [
  #     { name: "Ackermann",
  #       data: [
  #         { timestamp: 12312341234, duration: 42 },
  #         { timestamp: 12312341235, duration: 41 },
  #         { timestamp: 12312341236, duration: 40 }
  #       ]
  #     },
  #     { name: "Fasta",
  #       data: [
  #         { timestamp: 12312341234, duration: 42 },
  #         { timestamp: 12312341235, duration: 41 },
  #         { timestamp: 12312341236, duration: 40 }
  #       ]
  #     }
  #   ]
  def to_results_json
    convenient_data = {}.tap do |data|
      BenchmarkResult
        .where(benchmark_batch_id: benchmark_batches.map(&:id))
        .includes(:benchmark_info)
        .includes(:benchmark_batch)
        .includes(:benchmark_durations)
        .each do |benchmark_result|
        data
          .fetch(benchmark_result.benchmark_name) do
            data[benchmark_result.benchmark_name] = []
          end
          .push(benchmark_result.to_json)

        benchmark_result.to_json
      end
    end

    [].tap do |nicely_formatted_json|
      convenient_data.each do |name, data|
        nicely_formatted_json << { name: name, data: data }
      end
    end
  end

  def self.find_or_create_by_name(relation, name)
    relation.find_or_create_by(name: name)
  end
end

# == Schema Information
#
# Table name: benchmark_configurations
#
#  id                  :integer          not null, primary key
#  name                :string
#  language_id         :integer
#  runtime_id          :integer
#  operating_system_id :integer
#  created_at          :datetime
#  updated_at          :datetime
#
